;*********************************************************************************************
; Software License Agreement                                                                 *
; The software supplied herewith by Microchip Technology Incorporated                        *
; (the "Company") is intended and supplied to you, the Company's                             *
; customer, for use solely and exclusively on Microchip products.                            *
;                                                                                            *
; The software is owned by the Company and/or its supplier, and is                           *
; protected under applicable copyright laws. All rights are reserved.                        *
; Any use in violation of the foregoing restrictions may subject the                         *
; user to criminal sanctions under applicable laws, as well as to                            *
; civil liability for the breach of the terms and conditions of this                         *
; license.                                                                                   *
;                                                                                            *
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,                          *
; WHETHER EXPRESS, IMPLIED OR STATU-TORY, INCLUDING, BUT NOT LIMITED                         *
; TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A                                *
; PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,                          *
; IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR                                 *
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.                                          *
;                                                                                            *
;*********************************************************************************************
; File:  SoftwarePWM.asm																	 *
; This file generates 3 software PWM signals using the Timer0 interrupt.					 *
; ISR code is provided in this file that should be executed when							 *
; a Timer0 interrupt occurs.  An initialization routine is also provided					 *
; that configures the I/O pins associated with the PWM signals and 							 *
; enables Timer0.																			 *
;*********************************************************************************************

		list      p=12F615        ; list directive to define processor
		#include <p12F615.inc>    ; processor specific variable definitions
		#include <SoftwarePWM.h>
		errorlevel  -302          ; suppress message 302 from list file

	global Dutycycle0, Dutycycle1, Dutycycle2, PWMFlags, InitPWM, ISRPWM, IntCount
	

	UDATA
IntCount	res 1
Dutycycle0	res 1			; Holds duty cycle for PWM channel 0 
Dutycycle1	res 1 			; Holds duty cycle for PWM channel 1
Dutycycle2	res 1			; Holds duty cycle for PWM channel 2
w_temp		res 1			; variable used for context saving
status_temp	res 1			; variable used for context saving
pclath_temp	res 1			; variable used for context saving
PWMFlags	res 1			; variable used for software flags



	CODE 

ISRPWM
	movwf		w_temp			; save off current W register contents
	movf		STATUS,w		; move status register into W register
	movwf		status_temp		; save off contents of STATUS register

; Write TMR0 to setup next interrupt interval 
	movlw	.99
	movwf	TMR0
;State Machine for PWM starts from here
DecIntCount						;Decrement IntCount Register
	decfsz	IntCount,F			;decrement IntCount register and if it is zero then make the output pin high
	goto	Channel0			;if IntCount register is not zero then go to chk the dutycycle of the signal
; If IntCount is 0, then it is time to start a new PWM signal period.
BeginPeriod
	bsf		PWM0				; Set all PWM output pins high
	bsf		PWM1
	bsf		PWM2

	movlw	.32					; Initialize IntCount to 32
	movwf	IntCount
	bsf		BeginPWM			; Set flag for main software loop

	goto	ExitISR				; Goto end of ISR code


; If it is not the beginning of the PWM period, we need to compare each 
; dutycycle to the value of IntCount.  This is done by performing a 
; subtraction and checking to see whether the result is 0.  When a match
; occurs, the output pin for the PWM channel is set to 0.
Channel0
	movf	Dutycycle0,W
	subwf	IntCount,W
	IFNZ						; Is IntCount - DutyCycle0 = 0?
	goto	Channel1
	bcf		PWM0				; Yes, set output pin to 0.

Channel1
	movf	Dutycycle1,W		
	subwf	IntCount,W			;Is IntCount - DutyCycle1 = 0?
	IFNZ
	goto	Channel2
	bcf		PWM1				;Yes, set output pin to 0

Channel2
	movf	Dutycycle2,W			
	subwf	IntCount,W			;Is IntCount - DutyCycle2 = 0?
	IFZ
	bcf		PWM2				;Yes, set output pin to 0

	
;ISR RestoreData
ExitISR
	bcf  		INTCON,T0IF			;clear the T0IF bit in the INTCON register
	movf		status_temp,w		; retrieve copy of STATUS register
	movwf		STATUS				; restore pre-isr STATUS register contents
	swapf		w_temp,f
	swapf		w_temp,w			; restore pre-isr W register contents
	retfie							; return from interrupt


;*****************************************************************************************************
;
;
InitPWM						;Initialization for software PWM 	

	BANK1					;Select the RAM memory bank 1

		
	;Timer0 assignment
	movlw	b'01000001'		;Select the internal clock & /4 prescaler
			 ;0-------		 GPIO Pull-up Enable bit
			 ;-1------		 Interrupt on rising edge of INT pin
			 ;--0-----		 T0CS->0. Select Timer mode and Internal instruction cycle clock (CLKOUT)
			 ;---0----		 T0SE->0. Select falling edge on T0CK1 pin
			 ;----0---		 PSA->0. Select Timer0 module for Prescaler
			 ;-----001		 PS2:PS0->001. Select 1:4 Prescaler rate
	movwf	OPTION_REG		;move the value of work register into OPTION register
	
	;GPIO assignment
	movlw	b'00001011'		;Make GP0, GP1 and GP3 as input and GP2, GP4, GP5 as output
	movwf	TRISIO

	CLRF	ANSEL			;Initialize ANSEL register for I/O port

	bcf		ANSEL,2			;Digital I/O
	bcf		ANSEL,3


	BANK0					;Select the RAM memory bank 0
	
	CLRF	GPIO			;Initialize I/O port
	clrf	CCP1CON
	
	;Interrupts Initialization
	movlw	b'10100000'		;Enable global and Timer0 interrupts
	movwf	INTCON			
	
	;Other Variable Initialization
	movlw	.32
	movwf	IntCount		; Initialize IntCount to 32
	
	;Variables to hold duty cycle values
	movlw	.00
	movwf	Dutycycle0
	movwf	Dutycycle1
	movwf	Dutycycle2
	
	return
	
	END
